@@ -168,7 +168,7 @@ sns.set(style="ticks")
 def get_bins(x,bw=0.05):
     return np.arange(min(x),max(x)+bw,bw)
 
-def UMI_hist(adata,x_hist='log10UMIs',groupby=None,show_groups='all',return_fig=None,**fig_para):
+def UMI_hist(adata,x_hist='log10UMIs',groupby=None, groups='all',return_fig=None,**fig_para):
     '''
     Visualize the log-UMI-amount distribution.
 
@@ -181,8 +181,8 @@ def UMI_hist(adata,x_hist='log10UMIs',groupby=None,show_groups='all',return_fig=
         The key of logUMI values stroed in adata.obs. The default is 'log10UMIs'.
     groupby : str
         The key of the droplet categories stored in adata.obs. 
-    show_groups : list of strings, optional
-        Droplet categories, e.g. ['Homo-ct1', 'Homo-ct2'] annotated in adata.obs[groupby] to display. 
+    groups : list of strings, optional
+        Droplet categories to visualize logUMI distributions. It should be eithor 'all' or a list of cell type annotations specified in adata.obs[groupby].
         The default is 'all'.
     return_fig : bool, optional
         Return the matplotlib figure. The default is None.
@@ -192,7 +192,7 @@ def UMI_hist(adata,x_hist='log10UMIs',groupby=None,show_groups='all',return_fig=
     Raises
     ------
     ValueError
-        If 'show_groups' specifies categories not matching with adata.obs[groupby].
+        If 'groups' specifies categories not matching with adata.obs[groupby].
 
     Returns
     -------
@@ -225,14 +225,14 @@ def UMI_hist(adata,x_hist='log10UMIs',groupby=None,show_groups='all',return_fig=
         col_list = sns.color_palette(palette, 1)
         plt.hist(obsdata[x_hist],bins,alpha=0.6, color=col_list[0])
     else:
-        if show_groups == 'all':
-            show_groups = obsdata[groupby].unique()
-        elif isinstance(show_groups,list) and len([1 for v in show_groups if v not in obsdata[groupby].unique()]):
+        if groups == 'all':
+            groups = obsdata[groupby].unique()
+        elif isinstance(groups,list) and len([1 for v in groups if v not in obsdata[groupby].unique()]):
             raise ValueError("'show_groups' contains values absent in 'groupby'.")
             
-        col_list = sns.color_palette(palette, len(show_groups))
-        v_list = [obsdata[x_hist][obsdata[groupby]==g] for g in show_groups]        
-        for i,g in enumerate(show_groups):
+        col_list = sns.color_palette(palette, len(groups))
+        v_list = [obsdata[x_hist][obsdata[groupby]==g] for g in groups]        
+        for i,g in enumerate(groups):
             plt.hist(v_list[i],bins,label=g,alpha=0.6, color=col_list[i])
 
         plt.legend(loc='upper right')
@@ -398,7 +398,7 @@ def volcano_2DE(de_df, marker_list=None, marker_colors=None, return_fig=None):
 
     if marker_list is not None:
         for idx in range(len(marker_list)):
-            plt.scatter(fc_y[marker_list[idx]], pval_y[marker_list[idx]],color=marker_colos[idx],s=20)
+            plt.scatter(fc_y[marker_list[idx]], pval_y[marker_list[idx]],color=marker_colors[idx],s=20)
             
     plt.plot([0,0],[0,323],c='black',linestyle='dashed')
     plt.plot([-6,6],[-np.log10(0.05),-np.log10(0.05)],c='black',linestyle='dashed')
